# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from truefoundry.pydantic_v1 import BaseModel, Field, StrictStr, validator


class TensorFlowFramework(BaseModel):
    """
    TensorFlowFramework
    """

    type: StrictStr = Field(default=..., description="Type of the framework")
    __properties = ["type"]

    @validator("type")
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ("tensorflow",):
            raise ValueError("must be one of enum values ('tensorflow')")
        return value

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TensorFlowFramework:
        """Create an instance of TensorFlowFramework from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TensorFlowFramework:
        """Create an instance of TensorFlowFramework from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TensorFlowFramework.parse_obj(obj)

        _obj = TensorFlowFramework.parse_obj({"type": obj.get("type")})
        return _obj
