# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

from truefoundry.ml._autogen.client.models.run_data_dto import RunDataDto
from truefoundry.ml._autogen.client.models.run_info_dto import RunInfoDto
from truefoundry.pydantic_v1 import BaseModel, Field


class RunDto(BaseModel):
    """
    RunDto
    """

    info: RunInfoDto = Field(...)
    data: Optional[RunDataDto] = None
    __properties = ["info", "data"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RunDto:
        """Create an instance of RunDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from truefoundry.pydantic_v1 by calling `to_dict()` of info
        if self.info:
            _dict["info"] = self.info.to_dict()
        # override the default output from truefoundry.pydantic_v1 by calling `to_dict()` of data
        if self.data:
            _dict["data"] = self.data.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RunDto:
        """Create an instance of RunDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RunDto.parse_obj(obj)

        _obj = RunDto.parse_obj(
            {
                "info": RunInfoDto.from_dict(obj.get("info"))
                if obj.get("info") is not None
                else None,
                "data": RunDataDto.from_dict(obj.get("data"))
                if obj.get("data") is not None
                else None,
            }
        )
        return _obj
