BUILDKIT_SECRET_MOUNT_PIP_CONF_ID = "pip.conf"
PIP_CONF_BUILDKIT_SECRET_MOUNT = (
    f"--mount=type=secret,id={BUILDKIT_SECRET_MOUNT_PIP_CONF_ID}"
)
PIP_CONF_SECRET_MOUNT_AS_ENV = (
    f"PIP_CONFIG_FILE=/run/secrets/{BUILDKIT_SECRET_MOUNT_PIP_CONF_ID}"
)

BUILDKIT_SECRET_MOUNT_UV_CONF_ID = "uv.toml"
UV_CONF_BUILDKIT_SECRET_MOUNT = (
    f"--mount=type=secret,id={BUILDKIT_SECRET_MOUNT_UV_CONF_ID}"
)
UV_CONF_SECRET_MOUNT_AS_ENV = (
    f"UV_CONFIG_FILE=/run/secrets/{BUILDKIT_SECRET_MOUNT_UV_CONF_ID}"
)

BUILDKIT_SECRET_MOUNT_UV_ENV_ID = "uv.env"
UV_ENV_BUILDKIT_SECRET_MOUNT = (
    f"--mount=type=secret,id={BUILDKIT_SECRET_MOUNT_UV_ENV_ID}"
)
UV_ENV_SECRET_MOUNT_AS_ENV = f". /run/secrets/{BUILDKIT_SECRET_MOUNT_UV_ENV_ID}"

BUILDKIT_SECRET_MOUNT_POETRY_ENV_ID = "poetry.env"
POETRY_ENV_BUILDKIT_SECRET_MOUNT = (
    f"--mount=type=secret,id={BUILDKIT_SECRET_MOUNT_POETRY_ENV_ID}"
)
POETRY_ENV_SECRET_MOUNT_AS_ENV = f". /run/secrets/{BUILDKIT_SECRET_MOUNT_POETRY_ENV_ID}"
