# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional, Union

from truefoundry.ml._autogen.client.models.stop import Stop
from truefoundry.pydantic_v1 import BaseModel, StrictFloat, StrictInt


class Parameters(BaseModel):
    """
    Parameters to pass to the model when generating  # noqa: E501
    """

    max_tokens: Optional[StrictInt] = None
    temperature: Optional[Union[StrictFloat, StrictInt]] = None
    top_k: Optional[Union[StrictFloat, StrictInt]] = None
    top_p: Optional[Union[StrictFloat, StrictInt]] = None
    stop: Optional[Stop] = None
    __properties = ["max_tokens", "temperature", "top_k", "top_p", "stop"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Parameters:
        """Create an instance of Parameters from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from truefoundry.pydantic_v1 by calling `to_dict()` of stop
        if self.stop:
            _dict["stop"] = self.stop.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Parameters:
        """Create an instance of Parameters from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Parameters.parse_obj(obj)

        _obj = Parameters.parse_obj(
            {
                "max_tokens": obj.get("max_tokens"),
                "temperature": obj.get("temperature"),
                "top_k": obj.get("top_k"),
                "top_p": obj.get("top_p"),
                "stop": Stop.from_dict(obj.get("stop"))
                if obj.get("stop") is not None
                else None,
            }
        )
        return _obj
