# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import re  # noqa: F401
import warnings
from typing import Optional

from truefoundry.ml._autogen.client.api_client import ApiClient
from truefoundry.ml._autogen.client.api_response import ApiResponse
from truefoundry.ml._autogen.client.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError,
)
from truefoundry.ml._autogen.client.models.body_get_search_runs_get import (
    BodyGetSearchRunsGet,
)
from truefoundry.ml._autogen.client.models.get_signed_urls_for_dataset_write_response_dto import (
    GetSignedURLsForDatasetWriteResponseDto,
)
from truefoundry.ml._autogen.client.models.run_response_dto import RunResponseDto
from truefoundry.ml._autogen.client.models.search_runs_response_dto import (
    SearchRunsResponseDto,
)
from truefoundry.pydantic_v1 import (
    StrictInt,
    StrictStr,
    conlist,
    validate_arguments,
)


class DeprecatedApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def get_run_by_name_get(
        self,
        run_name: StrictStr,
        experiment_id: Optional[StrictStr] = None,
        experiment_name: Optional[StrictStr] = None,
        **kwargs,
    ) -> RunResponseDto:  # noqa: E501
        """(Deprecated) Get Run By Name  # noqa: E501

        Get Run by run name  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_run_by_name_get(run_name, experiment_id, experiment_name, async_req=True)
        >>> result = thread.get()

        :param run_name: (required)
        :type run_name: str
        :param experiment_id:
        :type experiment_id: str
        :param experiment_name:
        :type experiment_name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: RunResponseDto
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the get_run_by_name_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.get_run_by_name_get_with_http_info(
            run_name, experiment_id, experiment_name, **kwargs
        )  # noqa: E501

    @validate_arguments
    def get_run_by_name_get_with_http_info(
        self,
        run_name: StrictStr,
        experiment_id: Optional[StrictStr] = None,
        experiment_name: Optional[StrictStr] = None,
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Get Run By Name  # noqa: E501

        Get Run by run name  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_run_by_name_get_with_http_info(run_name, experiment_id, experiment_name, async_req=True)
        >>> result = thread.get()

        :param run_name: (required)
        :type run_name: str
        :param experiment_id:
        :type experiment_id: str
        :param experiment_name:
        :type experiment_name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(RunResponseDto, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "GET /api/2.0mlflow/runs/get-by-name is deprecated.", DeprecationWarning
        )

        _params = locals()

        _all_params = ["run_name", "experiment_id", "experiment_name"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_run_by_name_get" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("run_name") is not None:  # noqa: E501
            _query_params.append(("run_name", _params["run_name"]))

        if _params.get("experiment_id") is not None:  # noqa: E501
            _query_params.append(("experiment_id", _params["experiment_id"]))

        if _params.get("experiment_name") is not None:  # noqa: E501
            _query_params.append(("experiment_name", _params["experiment_name"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "RunResponseDto",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0mlflow/runs/get-by-name",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def get_search_runs_get(
        self,
        filter: Optional[StrictStr] = None,
        page_token: Optional[StrictStr] = None,
        offset: Optional[StrictInt] = None,
        run_view_type: Optional[StrictStr] = None,
        max_results: Optional[StrictInt] = None,
        body_get_search_runs_get: Optional[BodyGetSearchRunsGet] = None,
        **kwargs,
    ) -> SearchRunsResponseDto:  # noqa: E501
        """(Deprecated) Get Search Runs  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_search_runs_get(filter, page_token, offset, run_view_type, max_results, body_get_search_runs_get, async_req=True)
        >>> result = thread.get()

        :param filter:
        :type filter: str
        :param page_token:
        :type page_token: str
        :param offset:
        :type offset: int
        :param run_view_type:
        :type run_view_type: str
        :param max_results:
        :type max_results: int
        :param body_get_search_runs_get:
        :type body_get_search_runs_get: BodyGetSearchRunsGet
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: SearchRunsResponseDto
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the get_search_runs_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.get_search_runs_get_with_http_info(
            filter,
            page_token,
            offset,
            run_view_type,
            max_results,
            body_get_search_runs_get,
            **kwargs,
        )  # noqa: E501

    @validate_arguments
    def get_search_runs_get_with_http_info(
        self,
        filter: Optional[StrictStr] = None,
        page_token: Optional[StrictStr] = None,
        offset: Optional[StrictInt] = None,
        run_view_type: Optional[StrictStr] = None,
        max_results: Optional[StrictInt] = None,
        body_get_search_runs_get: Optional[BodyGetSearchRunsGet] = None,
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Get Search Runs  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_search_runs_get_with_http_info(filter, page_token, offset, run_view_type, max_results, body_get_search_runs_get, async_req=True)
        >>> result = thread.get()

        :param filter:
        :type filter: str
        :param page_token:
        :type page_token: str
        :param offset:
        :type offset: int
        :param run_view_type:
        :type run_view_type: str
        :param max_results:
        :type max_results: int
        :param body_get_search_runs_get:
        :type body_get_search_runs_get: BodyGetSearchRunsGet
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(SearchRunsResponseDto, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "GET /api/2.0/preview/mlflow/runs/search is deprecated.", DeprecationWarning
        )

        _params = locals()

        _all_params = [
            "filter",
            "page_token",
            "offset",
            "run_view_type",
            "max_results",
            "body_get_search_runs_get",
        ]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_search_runs_get" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("filter") is not None:  # noqa: E501
            _query_params.append(("filter", _params["filter"]))

        if _params.get("page_token") is not None:  # noqa: E501
            _query_params.append(("page_token", _params["page_token"]))

        if _params.get("offset") is not None:  # noqa: E501
            _query_params.append(("offset", _params["offset"]))

        if _params.get("run_view_type") is not None:  # noqa: E501
            _query_params.append(("run_view_type", _params["run_view_type"]))

        if _params.get("max_results") is not None:  # noqa: E501
            _query_params.append(("max_results", _params["max_results"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["body_get_search_runs_get"] is not None:
            _body_params = _params["body_get_search_runs_get"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "SearchRunsResponseDto",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/preview/mlflow/runs/search",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )

    @validate_arguments
    def get_signed_urls_for_dataset_write_deprecated_get(
        self, dataset_fqn: StrictStr, paths: conlist(StrictStr), **kwargs
    ) -> GetSignedURLsForDatasetWriteResponseDto:  # noqa: E501
        """(Deprecated) Get Signed Urls For Dataset Write Deprecated  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_signed_urls_for_dataset_write_deprecated_get(dataset_fqn, paths, async_req=True)
        >>> result = thread.get()

        :param dataset_fqn: (required)
        :type dataset_fqn: str
        :param paths: (required)
        :type paths: List[str]
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetSignedURLsForDatasetWriteResponseDto
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the get_signed_urls_for_dataset_write_deprecated_get_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.get_signed_urls_for_dataset_write_deprecated_get_with_http_info(
            dataset_fqn, paths, **kwargs
        )  # noqa: E501

    @validate_arguments
    def get_signed_urls_for_dataset_write_deprecated_get_with_http_info(
        self, dataset_fqn: StrictStr, paths: conlist(StrictStr), **kwargs
    ) -> ApiResponse:  # noqa: E501
        """(Deprecated) Get Signed Urls For Dataset Write Deprecated  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_signed_urls_for_dataset_write_deprecated_get_with_http_info(dataset_fqn, paths, async_req=True)
        >>> result = thread.get()

        :param dataset_fqn: (required)
        :type dataset_fqn: str
        :param paths: (required)
        :type paths: List[str]
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetSignedURLsForDatasetWriteResponseDto, status_code(int), headers(HTTPHeaderDict))
        """

        warnings.warn(
            "GET /api/2.0/mlflow/mlfoundry-artifacts/datasets/get-signed-urls-for-write is deprecated.",
            DeprecationWarning,
        )

        _params = locals()

        _all_params = ["dataset_fqn", "paths"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_signed_urls_for_dataset_write_deprecated_get" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("dataset_fqn") is not None:  # noqa: E501
            _query_params.append(("dataset_fqn", _params["dataset_fqn"]))

        if _params.get("paths") is not None:  # noqa: E501
            _query_params.append(("paths", _params["paths"]))
            _collection_formats["paths"] = "multi"

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # authentication setting
        _auth_settings = ["APIKey"]  # noqa: E501

        _response_types_map = {
            "200": "GetSignedURLsForDatasetWriteResponseDto",
            "422": "HTTPValidationError",
        }

        return self.api_client.call_api(
            "/api/2.0/mlflow/mlfoundry-artifacts/datasets/get-signed-urls-for-write",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )
