# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import json
import re  # noqa: F401

from aenum import Enum


class MimeType(str, Enum):
    """
    MIME type of the content
    """

    """
    allowed enum values
    """
    TEXT_SLASH_PLAIN = "text/plain"
    APPLICATION_SLASH_JSON = "application/json"
    IMAGE_SLASH_PNG = "image/png"
    IMAGE_SLASH_JPEG = "image/jpeg"
    APPLICATION_SLASH_X_MINUS_DIRECTORY = "application/x-directory"

    @classmethod
    def from_json(cls, json_str: str) -> MimeType:
        """Create an instance of MimeType from a JSON string"""
        return MimeType(json.loads(json_str))
