# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

from truefoundry.pydantic_v1 import BaseModel, Field, StrictInt, StrictStr


class RunLogInputDto(BaseModel):
    """
    RunLogInputDto
    """

    key: StrictStr = Field(...)
    value: Optional[StrictStr] = None
    timestamp: StrictInt = Field(...)
    step: Optional[StrictInt] = 0
    log_type: StrictStr = Field(...)
    artifact_path: Optional[StrictStr] = None
    __properties = ["key", "value", "timestamp", "step", "log_type", "artifact_path"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RunLogInputDto:
        """Create an instance of RunLogInputDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RunLogInputDto:
        """Create an instance of RunLogInputDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RunLogInputDto.parse_obj(obj)

        _obj = RunLogInputDto.parse_obj(
            {
                "key": obj.get("key"),
                "value": obj.get("value"),
                "timestamp": obj.get("timestamp"),
                "step": obj.get("step") if obj.get("step") is not None else 0,
                "log_type": obj.get("log_type"),
                "artifact_path": obj.get("artifact_path"),
            }
        )
        return _obj
