# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import json
import re  # noqa: F401

from aenum import Enum


class ArtifactType(str, Enum):
    """
    An enumeration.
    """

    """
    allowed enum values
    """
    ARTIFACT = "artifact"
    MODEL = "model"
    PLOT = "plot"
    IMAGE = "image"
    CHAT_PROMPT = "chat_prompt"
    OPENAPI_MINUS_TOOL = "openapi-tool"
    AGENT = "agent"

    @classmethod
    def from_json(cls, json_str: str) -> ArtifactType:
        """Create an instance of ArtifactType from a JSON string"""
        return ArtifactType(json.loads(json_str))
