# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import json
import re  # noqa: F401

from aenum import Enum


class Method(str, Enum):
    """
    HTTP Method for the tool call HTTP request
    """

    """
    allowed enum values
    """
    GET = "get"
    POST = "post"
    PUT = "put"
    DELETE = "delete"
    PATCH = "patch"

    @classmethod
    def from_json(cls, json_str: str) -> Method:
        """Create an instance of Method from a JSON string"""
        return Method(json.loads(json_str))
