# coding: utf-8

"""
TrueFoundry ML API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import json
import re  # noqa: F401

from aenum import Enum


class LibraryName(str, Enum):
    """
    Name of the library for the framework
    """

    """
    allowed enum values
    """
    TRANSFORMERS = "transformers"
    SENTENCE_MINUS_TRANSFORMERS = "sentence-transformers"
    DIFFUSERS = "diffusers"

    @classmethod
    def from_json(cls, json_str: str) -> LibraryName:
        """Create an instance of LibraryName from a JSON string"""
        return LibraryName(json.loads(json_str))
